SECRET_KEY = 'test-key'
CACHES = {
    'default': {
        'BACKEND': 'django.core.cache.backends.dummy.DummyCache',
    }
}
INSTALLED_APPS = [
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'permissions_auditor',
]
ROOT_URLCONF = None



from permissions_auditor.processors import auth_mixins
from permissions_auditor.tests import test_views
from permissions_auditor.tests.base import ProcessorTestCase


class MixinProcessorTestCaseMixin:
    """Mixins should never be able to process function based views."""

    def test_base_view(self):
        self.assertProcessorResults(test_views.base_view, can_process=False)

    def test_login_required_view(self):
        self.assertProcessorResults(test_views.login_required_view, can_process=False)

    def test_permission_required_view(self):
        self.assertProcessorResults(test_views.permission_required_view, can_process=False)

    def test_staff_member_required_view(self):
        self.assertProcessorResults(test_views.staff_member_required_view, can_process=False)

    def test_active_user_required_view(self):
        self.assertProcessorResults(test_views.active_user_required_view, can_process=False)

    def test_anonymous_user_required_view(self):
        self.assertProcessorResults(test_views.anonymous_user_required_view, can_process=False)

    def test_superuser_required_view(self):
        self.assertProcessorResults(test_views.superuser_required_view, can_process=False)

    def test_user_passes_test_view(self):
        self.assertProcessorResults(test_views.user_passes_test_view, can_process=False)


class TestLoginRequiredMixinProcessor(MixinProcessorTestCaseMixin, ProcessorTestCase):

    def setUp(self):
        self.processor = auth_mixins.LoginRequiredMixinProcessor()

    def test_cb_baseview(self):
        self.assertProcessorResults(test_views.BaseView, can_process=False)

    def test_cb_loginrequiredview(self):
        self.assertProcessorResults(
            test_views.LoginRequiredView,
            can_process=True,
            permissions=[],
            login_required=True,
            docstring=None
        )

    def test_cb_permissionsrequiredview(self):
        self.assertProcessorResults(test_views.PermissionRequiredView, can_process=False)

    def test_cb_userpassestestview(self):
        self.assertProcessorResults(test_views.UserPassesTestView, can_process=False)


class TestPermissionRequiredMixinProcessor(MixinProcessorTestCaseMixin, ProcessorTestCase):

    def setUp(self):
        self.processor = auth_mixins.PermissionRequiredMixinProcessor()

    def test_cb_baseview(self):
        self.assertProcessorResults(test_views.BaseView, can_process=False)

    def test_cb_loginrequiredview(self):
        self.assertProcessorResults(test_views.LoginRequiredView, can_process=False)

    def test_cb_permissionsrequiredview(self):
        self.assertProcessorResults(
            test_views.PermissionRequiredView,
            can_process=True,
            permissions=['tests.test_perm'],
            login_required=True,
            docstring=None
        )

    def test_cb_permissionsrequiredview_docstring(self):
        """Views that implement has_permission() and have a docstring should be retrieved."""
        self.assertProcessorResults(
            test_views.PermissionRequiredViewDocstring,
            can_process=True,
            permissions=['tests.test_perm'],
            login_required=True,
            docstring='Custom docstrings should be detected.'
        )

    def test_cb_permissionsrequiredview_no_docstring(self):
        """
        Views that implement has_permission() and do not have a docstring
        should return a default messsage.
        """
        self.assertProcessorResults(
            test_views.PermissionRequiredViewNoDocstring,
            can_process=True,
            permissions=['tests.test_perm'],
            login_required=True,
            docstring='Custom (no docstring found)'
        )

    def test_cb_permissionrequiredview_multi(self):
        """
        Views with multiple permissions should return all permissions.
        """
        self.assertProcessorResults(
            test_views.PermissionRequiredMultiView,
            can_process=True,
            permissions=['tests.test_perm', 'tests.test_perm2'],
            login_required=True,
            docstring=None
        )

    def test_cb_userpassestestview(self):
        self.assertProcessorResults(test_views.UserPassesTestView, can_process=False)


class TestUserPassesTestMixinProcessor(MixinProcessorTestCaseMixin, ProcessorTestCase):

    def setUp(self):
        self.processor = auth_mixins.UserPassesTestMixinProcessor()

    def test_cb_baseview(self):
        self.assertProcessorResults(test_views.BaseView, can_process=False)

    def test_cb_loginrequiredview(self):
        self.assertProcessorResults(test_views.LoginRequiredView, can_process=False)

    def test_cb_userpassestestview(self):
        self.assertProcessorResults(
            test_views.UserPassesTestView,
            can_process=True,
            permissions=[],
            login_required=None,
            docstring='Custom (no docstring found)'
        )

    def test_cb_permissionsrequiredview(self):
        self.assertProcessorResults(test_views.PermissionRequiredView, can_process=False)

    def test_cb_userpassestestview_docstring(self):
        """Views that implement test_func() and have a docstring should be retrieved."""
        self.assertProcessorResults(
            test_views.UserPassesTestViewDocstring,
            can_process=True,
            permissions=[],
            login_required=None,
            docstring='Custom docstrings should be detected.'
        )

    def test_cb_userpassestestview_no_docstring(self):
        """
        Views that implement test_func() and do not have a docstring
        should return a default messsage.
        """
        self.assertProcessorResults(
            test_views.UserPassesTestViewNoDocstring,
            can_process=True,
            permissions=[],
            login_required=None,
            docstring='Custom (no docstring found)'
        )

    def test_cb_userpassestestview_custom_func(self):
        """
        Views that override get_test_func() should check the new function returned
        instead of the default test_func() function.
        """
        self.assertProcessorResults(
            test_views.UserPassesTestViewCustomFunc,
            can_process=True,
            permissions=[],
            login_required=None,
            docstring='Custom docstrings should be detected.'
        )



from collections import namedtuple

from django.conf import ImproperlyConfigured, settings
from django.contrib.admindocs.views import simplify_regex
from django.core.cache import cache
from django.urls.resolvers import RegexPattern, RoutePattern, URLPattern, URLResolver
from django.utils.module_loading import import_string

from . import defaults


def _get_setting(name):
    return getattr(settings, name, getattr(defaults, name))


def _get_blacklist(name):
    blacklist = _get_setting('PERMISSIONS_AUDITOR_BLACKLIST')

    # Fall back to the defauls if the user does not provide the specific
    # blacklist in their settings.
    if name not in blacklist:
        blacklist = getattr(defaults, 'PERMISSIONS_AUDITOR_BLACKLIST')

    return blacklist[name]


def _get_cache_key(base_url):
    prefix = _get_setting('PERMISSIONS_AUDITOR_CACHE_KEY')
    return '{}.{}'.format(prefix, base_url)


NAMESPACE_BLACKLIST = tuple(_get_blacklist('namespaces'))
VIEW_BLACKLIST = tuple(_get_blacklist('view_names'))
MODULE_BLACKLIST = tuple(_get_blacklist('modules'))

ViewDetails = namedtuple('ViewDetails', [
    'module', 'name', 'url', 'permissions', 'login_required', 'docstring'
])


class ViewParser:

    def __init__(self):
        self.load_processors()

    def load_processors(self):
        self._processors = []

        for processor_path in _get_setting('PERMISSIONS_AUDITOR_PROCESSORS'):

            try:
                processor = import_string(processor_path)
                self._processors.append(processor())
            except (ImportError, TypeError):
                raise ImproperlyConfigured(
                    '{} is not a valid permissions processor.'.format(processor_path)
                )

    def parse(self, view):
        """
        Process a view.

        Returns a tuple containing:
        permissions (list), login_required (boolean or None), docstring (str)
        """
        permissions = []
        login_required = False
        docstrings = []

        for processor in self._processors:
            if processor.can_process(view):
                permissions.extend(processor.get_permission_required(view))

                login_required_result = processor.get_login_required(view)
                if login_required_result is None and not login_required:
                    login_required = login_required_result
                else:
                    login_required = login_required_result or login_required

                docstrings.append(processor.get_docstring(view))

        return permissions, login_required, '\n'.join(list(set(filter(None, docstrings))))


def get_all_views(urlpatterns=None, base_url=''):
    """
    Get all views in the specified urlpatterns.

    If urlpatterns is not specified, uses the `PERMISSIONS_AUDITOR_ROOT_URLCONF`
    setting, which by default is the value of `ROOT_URLCONF` in your project settings.

    Returns a list of `View` namedtuples.
    """
    cache_key = _get_cache_key(base_url)
    cache_timeout = _get_setting('PERMISSIONS_AUDITOR_CACHE_TIMEOUT')

    views = cache.get(cache_key)

    if views is None:
        views = []

        if urlpatterns is None:
            root_urlconf = __import__(_get_setting('PERMISSIONS_AUDITOR_ROOT_URLCONF'))
            urlpatterns = root_urlconf.urls.urlpatterns

        parser = ViewParser()

        for pattern in urlpatterns:
            if isinstance(pattern, RoutePattern) or isinstance(pattern, URLResolver):

                if pattern.namespace in NAMESPACE_BLACKLIST:
                    continue

                # Recursively fetch patterns
                views.extend(get_all_views(pattern.url_patterns, base_url + str(pattern.pattern)))

            elif isinstance(pattern, URLPattern) or isinstance(pattern, RegexPattern):
                view = pattern.callback

                # If this is a CBV, use the actual class instead of the as_view() classmethod.
                view = getattr(view, 'view_class', view)

                full_view_path = '{}.{}'.format(view.__module__, view.__name__)
                if full_view_path in VIEW_BLACKLIST or view.__module__ in MODULE_BLACKLIST:
                    continue

                permissions, login_required, docstring = parser.parse(view)

                views.append(ViewDetails._make([
                    view.__module__,
                    view.__name__,
                    simplify_regex(base_url + str(pattern.pattern)),
                    permissions,
                    login_required,
                    docstring
                ]))

        cache.set(cache_key, views, cache_timeout)

    return views



import inspect

from django.conf import ImproperlyConfigured


class BaseProcessor:

    def can_process(self, view):
        """
        Can this processor process the provided view?
        Useful when implementing filtering.
        """
        return True

    def get_permission_required(self, view):
        """
        Return the permissions required on the provided view.
        Must return an iterable.
        """
        return []

    def get_login_required(self, view):
        """
        Return True if the provided view requires the user to be loged in.
        """
        return False

    def get_docstring(self, view):
        """
        Return any additional information that should be displayed when
        showing permisison information.
        """
        return None


class BaseFuncViewProcessor(BaseProcessor):
    """Base class for function based views."""

    def can_process(self, view):
        return inspect.isfunction(view)


class BaseMixinProcessor(BaseProcessor):
    """Base class for parsing mixins on class based views."""

    def can_process(self, view):
        return inspect.isclass(view)


class BaseFileredMixinProcessor(BaseMixinProcessor):
    """
    Base class for parsing mixins on class based views.
    Override `class_filter` to filter the class names the processor applies to.
    ONLY checks top level base classes.
    """
    class_filter = None

    def can_process(self, view):
        if not super().can_process(view):
            return False

        view_bases = [cls.__module__ + '.' + cls.__name__ for cls in view.__bases__]

        for cls_filter in self.get_class_filter(view):
            if cls_filter in view_bases:
                return True

        return False

    def get_class_filter(self, view):
        """
        Override this method to override the class_names attribute.
        Must return an iterable.
        """
        if self.class_filter is None:
            raise ImproperlyConfigured(
                '{0} is missing the class_filter attribute. Define {0}.class_filter, or override '
                '{0}.get_class_filter().'.format(self.__class__.__name__)
            )
        if isinstance(self.class_filter, str):
            cls_filter = (self.class_filter, )
        else:
            cls_filter = self.class_filter
        return cls_filter



import os
from setuptools import find_packages, setup

with open(os.path.join(os.path.dirname(__file__), 'README.md')) as readme:
    README = readme.read()


setup(
    name='django-permissions-auditor',
    version=__import__('permissions_auditor').__version__,
    description='django-permissions-auditor is a tool to audit access control on your django app.',
    long_description=README,
    author='AAC Engineering',
    url='https://github.com/AACEngineering/django-permissions-auditor',
    license='MIT',
    packages=find_packages(exclude=['example']),
    include_package_data=True,
    install_requires=[
        'django>=2.1'
        'setuptools'
    ],
    zip_safe=False,
    test_suite='runtests.runtests',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Framework :: Django',
        'Framework :: Django :: 2.1',
        'Topic :: Internet :: WWW/HTTP :: Site Management',
        'Topic :: Internet :: WWW/HTTP :: Site Management :: Link Checking',
    ],
    keywords='django,admin,permissions,audit,auditor',
)
